package Pcore::Ext::App;

use Pcore -class;

1;
__END__
=pod

=encoding utf8

=head1 NAME

Pcore::Ext::App

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 ATTRIBUTES

=head1 METHODS

=head1 SEE ALSO

=cut
